using System;
using System.Data;
using Vevo.Shared.DataAccess;

namespace Vevo.DataAccessLib
{
    public static class PageAccess
    {
        private const string GlobalPathText = "Global (Can be used in any page)";

        private static string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy + " ";
        }


        private static bool IsExistingData(
            string pageID,
            string path )
        {
            string count = DataAccess.ExecuteScalar(
                "SELECT COUNT(*) " +
                "FROM Page " +
                "WHERE PageID = @PageID OR Path = @Path",
                DataAccess.CreateParameterString( pageID ),
                DataAccess.CreateParameterString( path ) );

            return (int.Parse( count ) > 0);
        }

        public static void Create(
            string pageID,
            string path )
        {
            if (IsExistingData( pageID, path ))
                throw new DuplicatedPrimaryKeyException(
                    "Table: Page, PageID: " + pageID );

            DataAccess.ExecuteNonQuery(
                "INSERT INTO Page (PageID, Path) " +
                "VALUES (@PageID, @Path); ",
                DataAccess.CreateParameterString( pageID ),
                DataAccess.CreateParameterString( path ) );
        }

        public static void Update(
            string pageID,
            string newPageID,
            string newPath )
        {
            if (pageID != newPageID &&
                IsExistingData( newPageID, newPath ))
                throw new DuplicatedPrimaryKeyException(
                    "Table: Page, PageID: " + pageID );

            DataAccess.ExecuteNonQuery(
                "UPDATE Page " +
                "SET PageID = @NewPageID, Path = @NewPath " +
                "WHERE PageID = @PageID; ",
                DataAccess.CreateParameterString( newPageID ),
                DataAccess.CreateParameterString( newPath ),
                DataAccess.CreateParameterString( pageID ) );

            DataAccess.ExecuteNonQuery(
                "UPDATE LanguageText " +
                "SET PageID = @NewPageID " +
                "WHERE PageID = @OldPageID; ",
                DataAccess.CreateParameterString( newPageID ),
                DataAccess.CreateParameterString( pageID ) );
        }

        public static void Delete( string pageID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM Page " +
                "WHERE PageID = @PageID; ",
                DataAccess.CreateParameterString( pageID ) );

            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM LanguageText " +
                "WHERE PageID = @PageID; ",
                DataAccess.CreateParameterString( pageID ) );
        }

        public static DataTable GetAll( string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT PageID, Path " +
                "FROM Page " +
                CreateSortByText( sortBy ) + "; " );

            //DataRow row = table.NewRow();
            //row["PageID"] = 0;
            //row["Path"] = GlobalPathText;
            //table.Rows.InsertAt( row, 0 );

            return table;
        }

        public static string GetIDByPath( string absolutePath )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT PageID " +
                "FROM Page " +
                "WHERE Path = @Path; ",
                DataAccess.CreateParameterString( absolutePath ) );

            if (table.Rows.Count > 0)
                return table.Rows[0]["PageID"].ToString();
            else
                return "0";
        }

        public static string GetPathByID( string pageID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT Path " +
                "FROM Page " +
                "WHERE PageID = @PageID; ",
                DataAccess.CreateParameterString( pageID ) );

            if (table.Rows.Count > 0)
                return table.Rows[0]["Path"].ToString();
            else
            {
                if (pageID == "0")
                    return GlobalPathText;
                else
                    return null;
            }
        }
    }
}
