using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.DataAccessLib;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class CategoryRepositoryNoCache : ICategoryRepository
    {
        #region Private
        private const string AllCategoryFields =
            "Category.CategoryID, ImageFile, ParentCategoryID, Category.IsEnabled, " +
            "SortOrder, DiscountGroupID, Other1, Other2, Other3, Other4, Other5, " +
            "CategoryListLayoutPath, ProductListLayoutPath, RootID, IsAnchor, IsShowNewArrival, NewArrivalAmount ";

        private const string AllCategoryLocaleFields =
            "CategoryID, CultureID, Name, Description, MetaKeyword, MetaDescription, ImageAlt, ImageTitle, UrlName ";

        private const string AllCategoryFieldsWithLocale =
            "Category.CategoryID, Name, Description, ImageFile, ParentCategoryID, UrlName, Category.IsEnabled, " +
            "SortOrder, DiscountGroupID, Other1, Other2, Other3, Other4, Other5, IsAnchor, IsShowNewArrival, NewArrivalAmount, " +
            "CategoryListLayoutPath, ProductListLayoutPath, RootID, MetaKeyword, MetaDescription, ImageAlt, ImageTitle ";

        private static UrlNameAccessHelper _urlNameHelper = new UrlNameAccessHelper( "CategoryLocale", "CategoryID" );
        private CategoryHelperRepository _categoryHelperRepository = new CategoryHelperRepository( "Category" );

        private static string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "categoryid":
                    return "Category.CategoryID";

                default:
                    return fieldName;
            }
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "categoryid" ))
                return "ORDER BY Category." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "name" ))
                return "ORDER BY L." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "sortorder" ))
                return "ORDER BY Category." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private string GetCultureID( Culture culture )
        {
            return culture.CultureID;
        }

        private void DeleteCategoryLocales( string categoryID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM CategoryLocale " +
                "WHERE CategoryID = @CategoryID; ",
                DataAccess.CreateParameterString( categoryID ) );
        }

        private void InsertCategoryLocales( Category category )
        {
            foreach (BaseCategoryLocale locale in category.Locales)
            {
                locale.UrlName = _urlNameHelper.CreateLocaleUrlName( locale.Name );

                DataAccess.ExecuteNonQuery(
                    "INSERT INTO CategoryLocale ( CategoryID, CultureID, Name, Description, MetaKeyword, MetaDescription, ImageAlt, ImageTitle, UrlName ) " +
                    "VALUES ( @CategoryID, @CultureID, @Name, @Description, @MetaKeyword, @MetaDescription, @ImageAlt, @ImageTitle, @UrlName ); " +
                    "SELECT @@Identity; ",
                    DataAccess.CreateParameterString( category.CategoryID ),
                    DataAccess.CreateParameterString( locale.CultureID ),
                    DataAccess.CreateParameterString( locale.Name ),
                    DataAccess.CreateParameterString( locale.Description ),
                    DataAccess.CreateParameterString( locale.MetaKeyword ),
                    DataAccess.CreateParameterString( locale.MetaDescription ),
                    DataAccess.CreateParameterString( locale.ImageAlt ),
                    DataAccess.CreateParameterString( locale.ImageTitle ),
                    DataAccess.CreateParameterString( locale.UrlName ) );
            }
        }

        private Category Create( Category category )
        {
            int recCount = int.Parse( DataAccess.ExecuteScalar(
                "SELECT COUNT(*) FROM Category WHERE ParentCategoryID = @ParentCategoryID and RootID = @RootID;",
                DataAccess.CreateParameterString( category.ParentCategoryID ),
                DataAccess.CreateParameterString( category.RootID ) ) );

            string newID = DataAccess.ExecuteScalar(
                "INSERT INTO Category ( ImageFile, ParentCategoryID, SortOrder, DiscountGroupID, IsEnabled, " +
                " Other1, Other2, Other3, Other4, Other5, CategoryListLayoutPath, ProductListLayoutPath, RootID, IsAnchor, IsShowNewArrival, NewArrivalAmount ) " +
                "VALUES ( @Image, @ParentCategoryID, @SortOrder, @DiscountGroupID, @IsEnabled, " +
                " @Other1, @Other2, @Other3, @Other4, @Other5, @CategoryListLayoutPath, @ProductListLayoutPath, @RootID, @IsAnchor, @IsShowNewArrival, @NewArrivalAmount ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( category.ImageFile ),
                DataAccess.CreateParameterString( category.ParentCategoryID ),
                DataAccess.CreateParameterInt32( recCount++ ),
                DataAccess.CreateParameterInt32( category.DiscountGroupID ),
                DataAccess.CreateParameterBool( category.IsEnabled ),
                DataAccess.CreateParameterString( category.Other1 ),
                DataAccess.CreateParameterString( category.Other2 ),
                DataAccess.CreateParameterString( category.Other3 ),
                DataAccess.CreateParameterString( category.Other4 ),
                DataAccess.CreateParameterString( category.Other5 ),
                DataAccess.CreateParameterString( category.CategoryListLayoutPath ),
                DataAccess.CreateParameterString( category.ProductListLayoutPath ),
                DataAccess.CreateParameterString( category.RootID ),
                DataAccess.CreateParameterBool( category.IsAnchor ),
                DataAccess.CreateParameterBool( category.IsShowNewArrival ),
                DataAccess.CreateParameterInt32( category.NewArrivalAmount ) );

            category.CategoryID = newID;

            InsertCategoryLocales( category );

            return category;
        }

        private Category CreateWithIdentity( Category category )
        {
            int recCount = int.Parse( DataAccess.ExecuteScalar(
                "SELECT COUNT(*) FROM Category WHERE ParentCategoryID = @ParentCategoryID and RootID = @RootID;",
                DataAccess.CreateParameterString( category.ParentCategoryID ),
                DataAccess.CreateParameterString( category.RootID ) ) );

            string newID = DataAccess.ExecuteScalar(
                "SET IDENTITY_INSERT [dbo].[Category] ON " +
                "INSERT INTO Category (CategoryID, ImageFile, ParentCategoryID, SortOrder, DiscountGroupID, IsEnabled, " +
                " Other1, Other2, Other3, Other4, Other5, CategoryListLayoutPath, ProductListLayoutPath, RootID, IsAnchor, IsShowNewArrival, NewArrivalAmount ) " +
                "VALUES ( @CategoryID, @Image, @ParentCategoryID, @SortOrder, @DiscountGroupID, @IsEnabled, " +
                " @Other1, @Other2, @Other3, @Other4, @Other5, @CategoryListLayoutPath, @ProductListLayoutPath, @RootID, @IsAnchor, @IsShowNewArrival, @NewArrivalAmount ); " +
                "SET IDENTITY_INSERT [dbo].[Category] OFF ",
                DataAccess.CreateParameterString( category.CategoryID ),
                DataAccess.CreateParameterString( category.ImageFile ),
                DataAccess.CreateParameterString( category.ParentCategoryID ),
                DataAccess.CreateParameterInt32( recCount++ ),
                DataAccess.CreateParameterInt32( category.DiscountGroupID ),
                DataAccess.CreateParameterBool( category.IsEnabled ),
                DataAccess.CreateParameterString( category.Other1 ),
                DataAccess.CreateParameterString( category.Other2 ),
                DataAccess.CreateParameterString( category.Other3 ),
                DataAccess.CreateParameterString( category.Other4 ),
                DataAccess.CreateParameterString( category.Other5 ),
                DataAccess.CreateParameterString( category.CategoryListLayoutPath ),
                DataAccess.CreateParameterString( category.ProductListLayoutPath ),
                DataAccess.CreateParameterString( category.RootID ),
                DataAccess.CreateParameterBool( category.IsAnchor ),
                DataAccess.CreateParameterBool( category.IsShowNewArrival ),
                DataAccess.CreateParameterInt32( category.NewArrivalAmount ) );

            InsertCategoryLocales( category );

            return category;
        }

        private Category FindByCategoryID( IList<Category> allCategories, string categoryID )
        {
            foreach (Category category in allCategories)
            {
                if (category.CategoryID == categoryID)
                    return category;
            }

            return Category.Null;
        }

        private bool IsChildOf( IList<Category> allCategories, string childID, string parentID )
        {
            string currentID = childID;
            int iNumSearch = 0;

            while (currentID != parentID
                && currentID != "0")
            {
                Category category = FindByCategoryID( allCategories, currentID );
                if (category.IsNull)
                {
                    throw new DataAccessException(
                        "Cannot find category with a category ID " + currentID + "." );
                }

                iNumSearch++;
                if (iNumSearch > allCategories.Count)
                    throw new DataAccessException(
                        "Invalid parent-child category relationship. Possibly a circular list" );

                currentID = category.ParentCategoryID.ToString();
            }

            if (currentID == parentID)
                return true;
            else
                return false;
        }

        private Category Update( Category category )
        {
            IList<Category> allCategories = GetAll( category.DefaultCulture, "CategoryID" );
            if (IsChildOf( allCategories, category.ParentCategoryID, category.CategoryID ))
            {
                Category parent = FindByCategoryID( allCategories, category.ParentCategoryID );

                throw new DataAccessException(
                    "Category '" +
                    category.Name +
                    "' is already the parent of category '" +
                    parent.Name +
                    "'. Cannot be chosen as the parent." );
            }

            int mainUpdate = DataAccess.ExecuteNonQuery(
                "UPDATE Category " +
                "SET ImageFile = @ImageFile, ParentCategoryID = @ParentCategoryID, " +
                "    DiscountGroupID = @DiscountGroupID, IsEnabled = @IsEnabled, " +
                "    Other1 = @Other1, Other2 = @Other2, Other3 = @Other3, Other4 = @Other4, Other5 = @Other5, " +
                "    CategoryListLayoutPath = @CategoryListLayoutPath, ProductListLayoutPath = @ProductListLayoutPath, IsAnchor = @IsAnchor, IsShowNewArrival = @IsShowNewArrival, NewArrivalAmount = @NewArrivalAmount " +
                "WHERE CategoryID = @CategoryID; ",
                DataAccess.CreateParameterString( category.ImageFile ),
                DataAccess.CreateParameterString( category.ParentCategoryID ),
                DataAccess.CreateParameterInt32( category.DiscountGroupID ),
                DataAccess.CreateParameterBool( category.IsEnabled ),
                DataAccess.CreateParameterString( category.Other1 ),
                DataAccess.CreateParameterString( category.Other2 ),
                DataAccess.CreateParameterString( category.Other3 ),
                DataAccess.CreateParameterString( category.Other4 ),
                DataAccess.CreateParameterString( category.Other5 ),
                DataAccess.CreateParameterString( category.CategoryListLayoutPath ),
                DataAccess.CreateParameterString( category.ProductListLayoutPath ),
                DataAccess.CreateParameterBool( category.IsAnchor ),
                DataAccess.CreateParameterBool( category.IsShowNewArrival ),
                DataAccess.CreateParameterInt32( category.NewArrivalAmount ),
                DataAccess.CreateParameterString( category.CategoryID ) );

            if (mainUpdate == 0)
                throw new DataAccessException(
                    "Category Update Failed. Not Found category with ID: " + category.CategoryID +
                    ", Name: " + category.Name );

            category.Locales.Load();
            DeleteCategoryLocales( category.CategoryID );
            InsertCategoryLocales( category );

            return category;
        }

        private Category CreateOneCategoryFromRow( Culture defaultCulture, DataRow row )
        {
            Category category = new Category( defaultCulture );

            // Data from Category database table
            category.CategoryID = row["CategoryID"].ToString();
            category.ImageFile = row["ImageFile"].ToString();

            category.ParentCategoryID = row["ParentCategoryID"].ToString();
            category.DiscountGroupID = (int) row["DiscountGroupID"];
            category.IsEnabled = (bool) row["IsEnabled"];
            category.Other1 = row["Other1"].ToString();
            category.Other2 = row["Other2"].ToString();
            category.Other3 = row["Other3"].ToString();
            category.Other4 = row["Other4"].ToString();
            category.Other5 = row["Other5"].ToString();
            category.CategoryListLayoutPath = ConvertUtilities.ToString( row["CategoryListLayoutPath"] );
            category.ProductListLayoutPath = ConvertUtilities.ToString( row["ProductListLayoutPath"] );
            category.RootID = row["RootID"].ToString();
            category.IsAnchor = (bool) row["IsAnchor"];
            category.IsShowNewArrival = (bool) row["IsShowNewArrival"];
            category.NewArrivalAmount = (int) row["NewArrivalAmount"];

            // Data from CategoryLocale database table
            category.Name = row["Name"].ToString();
            category.Description = row["Description"].ToString();
            category.MetaKeyword = row["MetaKeyword"].ToString();
            category.MetaDescription = row["MetaDescription"].ToString();
            category.ImageAlt = row["ImageAlt"].ToString();
            category.ImageTitle = row["ImageTitle"].ToString();
            category.UrlName = row["UrlName"].ToString();
            return category;
        }

        private CategoryLocale CreateOneCategoryLocaleFromRow( DataRow row )
        {
            CategoryLocale locale = new CategoryLocale();
            locale.CultureID = row["CultureID"].ToString();
            locale.Name = row["Name"].ToString();
            locale.Description = row["Description"].ToString();
            locale.MetaKeyword = row["MetaKeyword"].ToString();
            locale.MetaDescription = row["MetaDescription"].ToString();
            locale.ImageAlt = row["ImageAlt"].ToString();
            locale.ImageTitle = row["ImageTitle"].ToString();
            locale.UrlName = row["UrlName"].ToString();
            return locale;
        }

        private List<ILocale> ConvertCategoryLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneCategoryLocaleFromRow( row ) );

            return list;
        }

        private IList<Category> ConvertCategoryTableToList( Culture defaultCulture, DataTable table )
        {
            IList<Category> list = new List<Category>();

            foreach (DataRow row in table.Rows)
                list.Add( CreateOneCategoryFromRow( defaultCulture, row ) );

            return list;
        }

        private IList<Category> GetOnlyIndex( IList<Category> categoryList, int startIndex, int endIndex )
        {
            IList<Category> list = new List<Category>();
            for (int i = startIndex; i <= endIndex && i < categoryList.Count; i++)
            {
                list.Add( categoryList[i] );
            }
            return list;
        }

        #endregion

        #region Constructors

        public CategoryRepositoryNoCache()
        {
        }

        #endregion

        #region Public Methods

        public Category Save( Category category )
        {
            Category result;
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (category.CategoryID == "0")
                    result = Create( category );
                else
                    result = Update( category );

                transaction.Complete();
            }

            return result;
        }

        public Category SaveWithIdentity( Category category )
        {
            Category result;
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                result = CreateWithIdentity( category );
                transaction.Complete();
            }

            return result;
        }

        public void Delete( string categoryID )
        {
            _categoryHelperRepository.Delete( categoryID );
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            _categoryHelperRepository.DeleteLocalesByCultureID( cultureID );
        }

        public void ReArrangeSortOrder( Culture culture, string parentCategoryID, string categoryID, BoolFilter isEnabled )
        {
            IList<Category> list = DataAccessContext.CategoryRepository.GetByParentID( culture, parentCategoryID, "SortOrder", isEnabled );
            int recCount = 0;
            string[] result = new string[list.Count + 1];
            foreach (Category category in list)
            {
                result[recCount] = category.CategoryID;
                recCount++;
            }
            result[recCount] = categoryID;
            DataAccessContext.CategoryRepository.UpdateSortOrder( result );
        }

        public void ReArrangeSortOrderByParentCategoryID( Culture culture, string parentCategoryID, BoolFilter isEnabled )
        {
            IList<Category> list = DataAccessContext.CategoryRepository.GetByParentID( culture,
                parentCategoryID, "sortOrder", isEnabled );
            string[] result = new string[list.Count];
            int recCount = 0;
            foreach (Category category in list)
            {
                result[recCount] = category.CategoryID;
                recCount++;
            }
            DataAccessContext.CategoryRepository.UpdateSortOrder( result );
        }

        public void UpdateSortOrder( string[] sortedCategoryIDs )
        {
            int sortCount = 0;
            foreach (string items in sortedCategoryIDs)
            {
                sortCount++;
                DataAccess.ExecuteNonQuery(
                    "UPDATE Category SET SortOrder = @SortOrder WHERE CategoryID = @CategoryID;",
                    DataAccess.CreateParameterString( sortCount.ToString() ),
                    DataAccess.CreateParameterString( items ) );
            }
        }

        public Category GetOne( Culture culture, string categoryID )
        {
            DataTable categoryTable = _categoryHelperRepository.GetOne( culture, categoryID );
            if (categoryTable.Rows.Count > 0)
                return CreateOneCategoryFromRow( culture, categoryTable.Rows[0] );
            else
                return Category.Null;
        }

        public Category GetOneByUrlName( Culture culture, string urlName )
        {
            DataTable table = _categoryHelperRepository.GetOneByUrlName( culture, urlName );

            if (table.Rows.Count > 0)
                return CreateOneCategoryFromRow( culture, table.Rows[0] );
            else
                return Category.Null;
        }

        public IList<Category> GetByParentID(
            Culture culture,
            string parentCategoryID,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetByParentID( culture, parentCategoryID, sortBy, isEnabled );
            DataColumn[] keys = new DataColumn[1] { table.Columns["CategoryID"] };
            table.PrimaryKey = keys;

            return ConvertCategoryTableToList( culture, table );
        }

        public IList<Category> GetByParentIDAndRootID(
            Culture culture,
            string parentCategoryID,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetByParentIDAndRootID( culture, parentCategoryID, rootID, sortBy, isEnabled );

            DataColumn[] keys = new DataColumn[1] { table.Columns["CategoryID"] };
            table.PrimaryKey = keys;

            return ConvertCategoryTableToList( culture, table );
        }

        public IList<Category> GetByParentID(
            Culture culture,
            string parentCategoryID,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            IList<Category> full = GetByParentID( culture, parentCategoryID, sortBy, isEnabled );

            howManyItems = full.Count;

            return GetOnlyIndex( full, startIndex, endIndex );
        }

        public IList<Category> GetByRootID(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetByRootID( culture, rootID, sortBy, isEnabled );

            DataColumn[] keys = new DataColumn[1] { table.Columns["CategoryID"] };
            table.PrimaryKey = keys;

            return ConvertCategoryTableToList( culture, table );
        }

        public IList<Category> GetByRootIDLeafOnly(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetByRootIDLeafOnly( culture, rootID, sortBy, isEnabled );

            DataColumn[] keys = new DataColumn[1] { table.Columns["CategoryID"] };
            table.PrimaryKey = keys;

            return ConvertCategoryTableToList( culture, table );
        }

        public IList<Category> GetByRootIDNotLeaf(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetByRootIDNotLeaf( culture, rootID, sortBy, isEnabled );

            DataColumn[] keys = new DataColumn[1] { table.Columns["CategoryID"] };
            table.PrimaryKey = keys;

            return ConvertCategoryTableToList( culture, table );
        }

        public IList<Category> GetByRootIDNoProduct(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetByRootIDNoProduct( culture, rootID, sortBy, isEnabled );

            DataColumn[] keys = new DataColumn[1] { table.Columns["CategoryID"] };
            table.PrimaryKey = keys;

            return ConvertCategoryTableToList( culture, table );
        }

        public IList<Category> GetByRootID(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            IList<Category> full = GetByRootID( culture, rootID, sortBy, isEnabled );

            howManyItems = full.Count;

            return GetOnlyIndex( full, startIndex, endIndex );
        }

        public IList<Category> GetRootCategory(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetRootCategory( culture, sortBy, isEnabled );

            DataColumn[] keys = new DataColumn[1] { table.Columns["CategoryID"] };
            table.PrimaryKey = keys;

            return ConvertCategoryTableToList( culture, table );
        }

        public IList<Category> GetRootCategory(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            IList<Category> full = GetRootCategory( culture, sortBy, isEnabled );

            howManyItems = full.Count;

            return GetOnlyIndex( full, startIndex, endIndex );
        }

        public IList<Category> GetByParentUrlName(
             Culture culture,
             string parentCategoryName,
             string sortBy,
             BoolFilter isEnabled,
             int startIndex,
             int endIndex,
             out int howManyItems )
        {
            DataTable full = _categoryHelperRepository.GetByParentUrlName( culture, parentCategoryName, sortBy, isEnabled, startIndex, endIndex );

            howManyItems = _categoryHelperRepository.GetCountByParentUrlName( culture, parentCategoryName, sortBy, isEnabled );
            return ConvertCategoryTableToList( culture, full );
        }

        public IList<Category> GetAll( Culture culture, string sortBy )
        {
            DataTable table = _categoryHelperRepository.GetAll( culture, sortBy );

            return ConvertCategoryTableToList( culture, table );
        }

        public IList<Category> GetAllLeafCategory( Culture culture, string sortBy )
        {
            DataTable table = _categoryHelperRepository.GetAllLeafOnly( culture, sortBy );

            return ConvertCategoryTableToList( culture, table );
        }

        public IList<Category> GetAll( Culture culture, string sortBy, BoolFilter isEnabled )
        {
            DataTable table = _categoryHelperRepository.GetAll( culture, sortBy, isEnabled );

            DataColumn[] keys = new DataColumn[1] { table.Columns["CategoryID"] };
            table.PrimaryKey = keys;
            return ConvertCategoryTableToList( culture, table );
        }

        public IList<Category> GetAll(
            Culture culture,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            IList<Category> full = GetAll( culture, sortBy );
            howManyItems = full.Count;
            return GetOnlyIndex( full, startIndex, endIndex );
        }

        public IList<Category> GetAllWithoutRootCategory( Culture culture, string sortBy )
        {
            DataTable table = _categoryHelperRepository.GetAllWithoutRootCategory( culture, sortBy );

            return ConvertCategoryTableToList( culture, table );
        }

        public IList<ILocale> GetLocales( string categoryID )
        {
            DataTable localeTable = _categoryHelperRepository.GetLocales( categoryID );

            return ConvertCategoryLocaleTableToList( localeTable );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            return _categoryHelperRepository.GetTableSchema();
        }

        public bool IsLeaf( string categoryID )
        {
            return _categoryHelperRepository.IsLeaf( categoryID );
        }

        public bool IsCategoryIDNotLeaf( string parentCategoryID )
        {
            return _categoryHelperRepository.IsNotLeaf( parentCategoryID );
        }

        public bool IsUrlNameNotLeaf( string parentCategoryUrlName )
        {
            return _categoryHelperRepository.IsUrlNameNotLeaf( parentCategoryUrlName );
        }

        public void RefreshUrlName( string categoryID, string name )
        {
            _categoryHelperRepository.RefreshUrlName( categoryID, name );
        }

        public IList<Category> SearchCategory(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter
            )
        {
            DataTable full = _categoryHelperRepository.Search( culture, sortBy, searchFilter );

            DataColumn[] keys = new DataColumn[] { full.Columns["CategoryID"] };
            full.PrimaryKey = keys;
            return ConvertCategoryTableToList( culture, full );
        }

        public IList<Category> SearchCategoryByRootID(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            string rootID
            )
        {
            DataTable full = _categoryHelperRepository.SearchByRootID( culture, sortBy, searchFilter, rootID );

            DataColumn[] keys = new DataColumn[] { full.Columns["CategoryID"] };
            full.PrimaryKey = keys;
            return ConvertCategoryTableToList( culture, full );
        }

        public IList<Category> SearchRootCategory(
           Culture culture,
           string sortBy,
           SearchFilter searchFilter
           )
        {
            DataTable full = _categoryHelperRepository.SearchRoot( culture, sortBy, searchFilter );

            DataColumn[] keys = new DataColumn[] { full.Columns["CategoryID"] };
            full.PrimaryKey = keys;
            return ConvertCategoryTableToList( culture, full );
        }

        public IList<string> GetLeafFromCategoryID( string parentCategoryID, IList<string> list )
        {
            if (_categoryHelperRepository.IsNotLeaf( parentCategoryID ))
            {
                DataTable table = DataAccess.ExecuteSelect(
                    " SELECT * FROM Category " +
                    " WHERE ParentCategoryID = " + parentCategoryID );

                foreach (DataRow row in table.Rows)
                    list.Add( row["CategoryID"].ToString() );

                IList<string> newList = new List<string>();
                for (int i = 0; i <= list.Count - 1; i++)
                {
                    string categoryID = list[i];
                    if (_categoryHelperRepository.IsNotLeaf( categoryID ))
                    {
                        list.RemoveAt( i );
                        DataAccessContext.CategoryRepository.GetLeafFromCategoryID( categoryID, list );
                    }
                }
            }
            else
            {
                list.Add( parentCategoryID );
            }

            return list;
        }

        public bool HasCategoryInDiscountGroup( string discountgroupID )
        {
            return _categoryHelperRepository.HasItInDiscountGroup( discountgroupID );
        }

        public int GetCountAllWithoutRootCategory()
        {
            return _categoryHelperRepository.GetCountAllWithoutRootCategoryDepartment();
        }

        #endregion
    }
}
