﻿using System.Collections.Generic;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ManufacturerRepository : IManufacturerRepository
    {
        #region Private

        private const string RepositoryName = " ManufactureRepository";

        private const string GetLocalesSignature = "GetLocales";
        private const string GetOneSignature = "GetOne";
        private const string GetManufacturerIDFromManufacturerNameSignature = "GetManufacturerIDFromManufacturerName";
        private const string GetAllSignature = "GetAll";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string SearchManufacturerSignature = "SearchManufacturer";

        private DataCache _manufacturerDataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { "ManufacturerLocale", "Manufacturer" } );

        private ManufacturerRepositoryNoCache _repository = new ManufacturerRepositoryNoCache();
        #endregion

        #region Constructors

        public ManufacturerRepository()
        {
        }

        public ManufacturerRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods
        public Manufacturer Save( Manufacturer manufacturer )
        {
            Manufacturer newManufacturer = _repository.Save( manufacturer );
            _manufacturerDataCache.ClearRepositoryCache();
            return newManufacturer;
        }

        public void Delete( string manufacturerID )
        {
            _repository.Delete( manufacturerID );
            _manufacturerDataCache.ClearRepositoryCache();
        }

        public void DeleteLocales( string manufacturerID )
        {
            _repository.DeleteLocales( manufacturerID );
            _manufacturerDataCache.ClearRepositoryCache();
        }

        public IList<ILocale> GetLocales( string manufacturerID )
        {
            IList<ILocale> list = (IList<ILocale>) _manufacturerDataCache.GetValue(
                 GetLocalesSignature,
                 _cacheDependency,
                 manufacturerID );

            if (list == null)
            {
                list = _repository.GetLocales( manufacturerID );

                _manufacturerDataCache.SetValue(
                    GetLocalesSignature,
                    list,
                    manufacturerID );
            }

            return list;
        }

        public void DeleteAll()
        {
            _repository.DeleteAll();
            _manufacturerDataCache.ClearRepositoryCache();
        }

        public Manufacturer GetOne( Culture culture, string manufacturerID )
        {
            Manufacturer manufacturer = (Manufacturer) _manufacturerDataCache.GetValue( GetOneSignature, _cacheDependency, culture.CultureID, manufacturerID );

            if (manufacturer == null)
            {
                manufacturer = _repository.GetOne( culture, manufacturerID );
                _manufacturerDataCache.SetValue( GetOneSignature, manufacturer, culture.CultureID, manufacturerID );
            }

            return manufacturer;
        }

        public string GetManufacturerIDFromManufacturerName( string manufacturerName )
        {
            string manufacturerID = (string) _manufacturerDataCache.GetValue( GetManufacturerIDFromManufacturerNameSignature, _cacheDependency, manufacturerName );

            if ((manufacturerID == string.Empty) || (manufacturerID == null))
            {
                manufacturerID = _repository.GetManufacturerIDFromManufacturerName( manufacturerName );
                _manufacturerDataCache.SetValue( GetManufacturerIDFromManufacturerNameSignature, manufacturerName );
            }

            return manufacturerID;
        }

        public IList<Manufacturer> GetAll( Culture culture, BoolFilter isEnabled, string sortBy )
        {
            IList<Manufacturer> list = (IList<Manufacturer>) _manufacturerDataCache.GetValue(
                 GetAllSignature,
                 _cacheDependency,
                 culture.CultureID, isEnabled, sortBy );

            if (list == null)
            {
                list = _repository.GetAll( culture, isEnabled, sortBy );

                _manufacturerDataCache.SetValue(
                    GetAllSignature,
                    list,
                    culture.CultureID, isEnabled, sortBy );
            }

            return list;
        }

        public IList<Manufacturer> GetAll(
            Culture culture,
             string sortBy,
             BoolFilter isEnabled,
             int startIndex,
             int endIndex,
             out int howManyItems )
        {
            Pair list = (Pair) _manufacturerDataCache.GetValue(
                GetAllSignature,
                _cacheDependency,
                culture.CultureID,
                sortBy,
                isEnabled,
                startIndex,
                endIndex );

            if (list == null)
            {
                list = new Pair();

                list.First = _repository.GetAll(
                    culture,
                    sortBy,
                    isEnabled,
                    startIndex,
                    endIndex,
                    out howManyItems );
                list.Second = howManyItems;

                _manufacturerDataCache.SetValue(
                    GetAllSignature,
                    list,
                    culture.CultureID,
                    sortBy,
                    isEnabled,
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (IList<Manufacturer>) list.First;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _manufacturerDataCache.GetValue( GetTableSchemaSignature, _cacheDependency );
            if (list == null)
            {
                list = _repository.GetTableSchema();
                _manufacturerDataCache.SetValue( GetTableSchemaSignature, list );
            }
            return list;
        }

        public IList<Manufacturer> SearchManufacturer( Culture culture, string sortBy, SearchFilter searchFilter,
            BoolFilter isActive, out int howManyItems )
        {
            Pair list = (Pair) _manufacturerDataCache.GetValue( SearchManufacturerSignature,
                _cacheDependency,
                culture.CultureID, sortBy, searchFilter, isActive );

            if (list == null)
            {
                list = new Pair();
                list.First = _repository.SearchManufacturer(
                    culture, sortBy, searchFilter, isActive, out howManyItems );
                list.Second = howManyItems;
                _manufacturerDataCache.SetValue(
                    SearchManufacturerSignature,
                    list,
                    culture.CultureID, sortBy, searchFilter, isActive );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (IList<Manufacturer>) list.First;
        }

        public void UpdateSortOrder( string[] sortedManufacturerIDs )
        {
            _repository.UpdateSortOrder( sortedManufacturerIDs );
            _manufacturerDataCache.ClearRepositoryCache();
        }

        public Manufacturer GetOneByUrlName( Culture culture, string urlName )
        {
            Manufacturer manufacturer = (Manufacturer) _manufacturerDataCache.GetValue( GetOneSignature, _cacheDependency, culture.CultureID, urlName );

            if (manufacturer == null)
            {
                manufacturer = _repository.GetOneByUrlName( culture, urlName );
                _manufacturerDataCache.SetValue( GetOneSignature, manufacturer, culture.CultureID, urlName );
            }

            return manufacturer;
        }

        #endregion
    }
}
