﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Data;
using Vevo.DataAccessLib;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ManufacturerRepositoryNoCache : IManufacturerRepository
    {
        #region Private
        private const string AllManufacturerFields =
            "Manufacturer.ManufacturerID, UrlName, SortOrder, IsEnabled, IsShowNewArrival, NewArrivalAmount ";

        private const string AllManufacturerLocaleFields =
            "ManufacturerID, CultureID, Name, ShortDescription, LongDescription, MetaKeyword, MetaDescription, ImageAlt, ImageTitle, ImageFile ";

        private const string AllManufacturerFieldsWithLocale =
            "Manufacturer.ManufacturerID, Name, ShortDescription, LongDescription, ImageFile, UrlName, IsEnabled, IsShowNewArrival , NewArrivalAmount, " +
            "SortOrder, MetaKeyword, MetaDescription, ImageAlt, ImageTitle ";

        private static UrlNameAccessHelper _urlNameHelper = new UrlNameAccessHelper( "Manufacturer", "ManufacturerID" );

        private Manufacturer Create( Manufacturer manufacturer )
        {
            int recCount = int.Parse( DataAccess.ExecuteScalar(
                "SELECT COUNT(*) FROM Manufacturer;" ) );

            string newID = DataAccess.ExecuteScalar(
                "INSERT INTO Manufacturer ( SortOrder, IsEnabled, IsShowNewArrival,NewArrivalAmount) " +
                "VALUES ( @SortOrder, @IsEnabled, @IsShowNewArrival,@NewArrivalAmount ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterInt32( recCount++ ),
                DataAccess.CreateParameterBool( manufacturer.IsEnabled ),
                DataAccess.CreateParameterBool( manufacturer.IsShowNewArrival ),
                DataAccess.CreateParameterInt32( manufacturer.NewArrivalAmount )
                );

            manufacturer.ManufacturerID = newID;

            manufacturer.UrlName = _urlNameHelper.UpdateUrlName(
                manufacturer.ManufacturerID.ToString(),
                manufacturer.Name );

            InsertManufacturerLocales( manufacturer );

            return manufacturer;
        }

        private void InsertManufacturerLocales( Manufacturer manufacturer )
        {
            foreach (ManufacturerLocale locale in manufacturer.Locales)
            {
                DataAccess.ExecuteNonQuery(
                    "INSERT INTO ManufacturerLocale ( ManufacturerID, CultureID, Name, ShortDescription, LongDescription, MetaKeyword, MetaDescription, ImageAlt, ImageTitle, ImageFile ) " +
                    "VALUES ( @ManufacturerID, @CultureID, @Name, @Description, @LongDescription, @MetaKeyword, @MetaDescription, @ImageAlt, @ImageTitle, @ImageFile ); " +
                    "SELECT @@Identity; ",
                    DataAccess.CreateParameterString( manufacturer.ManufacturerID ),
                    DataAccess.CreateParameterString( locale.CultureID ),
                    DataAccess.CreateParameterString( locale.Name ),
                    DataAccess.CreateParameterString( locale.Description ),
                    DataAccess.CreateParameterString( locale.LongDescription ),
                    DataAccess.CreateParameterString( locale.MetaKeyword ),
                    DataAccess.CreateParameterString( locale.MetaDescription ),
                    DataAccess.CreateParameterString( locale.ImageAlt ),
                    DataAccess.CreateParameterString( locale.ImageTitle ),
                    DataAccess.CreateParameterString( locale.ImageFile ) );
            }
        }

        private Manufacturer Update( Manufacturer manufacturer )
        {
            int mainUpdate = DataAccess.ExecuteNonQuery(
                "UPDATE Manufacturer " +
                "SET IsEnabled = @IsEnabled, IsShowNewArrival = @IsShowNewArrival, NewArrivalAmount = @NewArrivalAmount WHERE ManufacturerID = @ManufacturerID; ",
                DataAccess.CreateParameterBool( manufacturer.IsEnabled ),
                DataAccess.CreateParameterBool( manufacturer.IsShowNewArrival ),
                DataAccess.CreateParameterInt32( manufacturer.NewArrivalAmount ),
                DataAccess.CreateParameterString( manufacturer.ManufacturerID ) );

            if (mainUpdate == 0)
                throw new DataAccessException(
                    "Manufacturer Update Failed. Not Found manufacturer with ID: " + manufacturer.ManufacturerID +
                    ", Name: " + manufacturer.Name );

            manufacturer.Locales.Load();
            DeleteLocales( manufacturer.ManufacturerID );

            manufacturer.UrlName = _urlNameHelper.UpdateUrlName(
                manufacturer.ManufacturerID.ToString(),
                manufacturer.Name );
            InsertManufacturerLocales( manufacturer );

            return manufacturer;
        }

        private string GetCultureID( Culture culture )
        {
           return culture.CultureID;
        }

        private List<ILocale> ConvertManufacturerLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneManufacturerLocaleFromRow( row ) );

            return list;
        }

        private ManufacturerLocale CreateOneManufacturerLocaleFromRow( DataRow row )
        {
            ManufacturerLocale locale = new ManufacturerLocale();
            locale.CultureID = row["CultureID"].ToString();
            locale.Name = row["Name"].ToString();
            locale.Description = row["ShortDescription"].ToString();
            locale.LongDescription = row["LongDescription"].ToString();
            locale.MetaKeyword = row["MetaKeyword"].ToString();
            locale.MetaDescription = row["MetaDescription"].ToString();
            locale.ImageAlt = row["ImageAlt"].ToString();
            locale.ImageTitle = row["ImageTitle"].ToString();
            locale.ImageFile = row["ImageFile"].ToString();

            return locale;
        }

        private IList<Manufacturer> ConvertManufacturerTableToList( Culture defaultCulture, DataTable table )
        {
            IList<Manufacturer> list = new List<Manufacturer>();

            foreach (DataRow row in table.Rows)
                list.Add( CreateOneManufacturerFromRow( defaultCulture, row ) );

            return list;
        }

        private Manufacturer CreateOneManufacturerFromRow( Culture defaultCulture, DataRow row )
        {
            Manufacturer manufacturer = new Manufacturer( defaultCulture );

            // Data from Manufacturer database table
            manufacturer.ManufacturerID = row["ManufacturerID"].ToString();
            manufacturer.UrlName = row["UrlName"].ToString();
            manufacturer.IsEnabled = (bool) row["IsEnabled"];
            manufacturer.IsShowNewArrival = ConvertUtilities.ToBoolean( row["IsShowNewArrival"] );
            manufacturer.NewArrivalAmount = ConvertUtilities.ToInt32( row["NewArrivalAmount"].ToString() );
            manufacturer.SortOrder = (int) row["SortOrder"];

            // Data from Manufacturer database table
            manufacturer.Name = row["Name"].ToString();
            manufacturer.Description = row["ShortDescription"].ToString();
            manufacturer.LongDescription = row["LongDescription"].ToString();
            manufacturer.MetaKeyword = row["MetaKeyword"].ToString();
            manufacturer.MetaDescription = row["MetaDescription"].ToString();
            manufacturer.ImageAlt = row["ImageAlt"].ToString();
            manufacturer.ImageTitle = row["ImageTitle"].ToString();
            manufacturer.ImageFile = row["ImageFile"].ToString();

            return manufacturer;
        }

        private string CreateSortByText( string sortBy )
        {
            string field = "manufaturerid";
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( field ))
                return "ORDER BY Manufacturer." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "name" ))
                return "ORDER BY L.Name ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private string ConvertFieldName( string fieldName )
        {
            string field = "manufacturerid";
            if (fieldName.ToLower() == field)
            {
                return "Manufacturer.ManufacturerID";
            }
            else
            {
                return fieldName;
            }
        }

        #endregion

        #region Public Methods

        public Manufacturer Save( Manufacturer manufacturer )
        {
            Manufacturer result;
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (manufacturer.ManufacturerID == "0")
                    result = Create( manufacturer );
                else
                    result = Update( manufacturer );

                transaction.Complete();
            }

            return result;
        }

        public void Delete( string manufacturerID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                    "DELETE FROM Manufacturer WHERE ManufacturerID = @ManufacturerID; ",
                    DataAccess.CreateParameterString( manufacturerID )
                    );
                DeleteLocales( manufacturerID );
                transaction.Complete();
            }
        }

        public void DeleteLocales( string manufacturerID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM ManufacturerLocale " +
                "WHERE ManufacturerID = @ManufacturerID; ",
                DataAccess.CreateParameterString( manufacturerID ) );
        }

        public IList<ILocale> GetLocales( string manufacturerID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
                "SELECT " + AllManufacturerLocaleFields +
                "FROM ManufacturerLocale WHERE ManufacturerID = @ManufacturerID; ",
                DataAccess.CreateParameterString( manufacturerID ) );

            return ConvertManufacturerLocaleTableToList( localeTable );
        }

        public void DeleteAll()
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery( "DELETE FROM Manufacturer;" );
                DataAccess.ExecuteNonQuery( "DELETE FROM ManufacturerLocale;" );
                transaction.Complete();
            }
        }

        public Manufacturer GetOne( Culture culture, string manufacturerID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllManufacturerFieldsWithLocale + "FROM Manufacturer " +
                " LEFT JOIN (SELECT * FROM ManufacturerLocale WHERE CultureID = @CultureID) AS L  " +
                " ON Manufacturer.ManufacturerID = L.ManufacturerID " +
                "WHERE Manufacturer.ManufacturerID = @ID; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( manufacturerID ) );

            if (table.Rows.Count > 0)
                return CreateOneManufacturerFromRow( culture, table.Rows[0] );
            else
                return NullManufacturer.Instance;
        }

        public string GetManufacturerIDFromManufacturerName( string manufacturerName )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT ManufacturerID FROM ManufacturerLocale " +
                "WHERE Name = @Name ;",
                DataAccess.CreateParameterString( manufacturerName ) );

            if (table.Rows.Count > 0)
                return table.Rows[0]["ManufacturerID"].ToString();
            else
                return string.Empty;
        }

        public IList<Manufacturer> GetAll( Culture culture, BoolFilter isEnabled, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllManufacturerFieldsWithLocale +
                "FROM Manufacturer LEFT JOIN " +
                "     (SELECT * FROM ManufacturerLocale WHERE CultureID = @CultureID) AS L " +
                "ON Manufacturer.ManufacturerID = L.ManufacturerID " +
                FlagFilterHelper.CreateFlagFilterWithWhere( "Manufacturer.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( culture.CultureID )
                );

            return ConvertManufacturerTableToList( culture, table );
        }

        private int GetAllCount(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled )
        {
            string query = "SELECT ROW_NUMBER() over (" + CreateSortByText( sortBy ) + ") as rownum, * FROM ( " +
                "SELECT " + AllManufacturerFieldsWithLocale +
                "FROM Manufacturer INNER JOIN " +
                "     (SELECT * FROM ManufacturerLocale WHERE CultureID = @CultureID) AS L " +
                "ON Manufacturer.ManufacturerID = L.ManufacturerID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "Manufacturer.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                ") AS Manufacturer";

            string getCountManu = "with SearchResult as (" + query + ") " +
                " select COUNT(rownum) from SearchResult;";

            string countResult = DataAccess.ExecuteScalar( getCountManu,
                DataAccess.CreateParameterString( culture.CultureID ) );

            return ConvertUtilities.ToInt32( countResult );
        }

        public IList<Manufacturer> GetAll(
             Culture culture,
             string sortBy,
             BoolFilter isEnabled,
             int startIndex,
             int endIndex,
             out int howManyItems )
        {
            string query = "SELECT ROW_NUMBER() over (" + CreateSortByText( sortBy ) + ") as rownum, * FROM ( " +
                "SELECT " + AllManufacturerFieldsWithLocale +
                "FROM Manufacturer INNER JOIN " +
                "     (SELECT * FROM ManufacturerLocale WHERE CultureID = @CultureID) AS L " +
                "ON Manufacturer.ManufacturerID = L.ManufacturerID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "Manufacturer.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                ") AS Manufacturer";

            string getManuItemsQuery = "with SearchResult as (" + query + ") " +
                " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable full = DataAccess.ExecuteSelect( getManuItemsQuery,
                DataAccess.CreateParameterString( culture.CultureID ) );



            howManyItems = GetAllCount( culture, sortBy, isEnabled );

            return ConvertManufacturerTableToList( culture, full );
        }


        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                    "SELECT Manufacturer.ManufacturerID, Name, ShortDescription, LongDescription, ImageFile, UrlName, IsEnabled, " +
                    " MetaKeyword, MetaDescription, ImageAlt, ImageTitle " +
                    "FROM Manufacturer LEFT JOIN ManufacturerLocale L " +
                    "    ON Manufacturer.ManufacturerID = L.ManufacturerID WHERE 0 = 1; " );

            DataColumn[] keys = new DataColumn[] { table.Columns["ManufacturerID"] };
            table.PrimaryKey = keys;
            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public IList<Manufacturer> SearchManufacturer( Culture culture, string sortBy, SearchFilter searchFilter,
            BoolFilter isActive, out int howManyItems )
        {
            string fieldName = ConvertFieldName( searchFilter.FieldName );
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, fieldName, parameterList );

            string query = "SELECT " + AllManufacturerFieldsWithLocale +
                "FROM Manufacturer LEFT JOIN " +
                "     (SELECT * FROM ManufacturerLocale WHERE CultureID = @CultureID) AS L " +
                "     ON Manufacturer.ManufacturerID = L.ManufacturerID " +
                commandSearch +
                CreateSortByText( sortBy ) + "; ";

            DataTable full = DataAccess.ExecuteSelect( query,
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            DataColumn[] keys = new DataColumn[] { full.Columns["ManufacturerID"] };
            full.PrimaryKey = keys;

            howManyItems = full.Rows.Count;
            return ConvertManufacturerTableToList( culture, full );
        }

        public void UpdateSortOrder( string[] sortedManufacturerIDs )
        {
            int sortCount = 0;
            foreach (string items in sortedManufacturerIDs)
            {
                sortCount++;
                DataAccess.ExecuteNonQuery(
                    "UPDATE Manufacturer SET SortOrder = @SortOrder WHERE ManufacturerID = @ManufacturerID;",
                    DataAccess.CreateParameterString( sortCount.ToString() ),
                    DataAccess.CreateParameterString( items ) );
            }
        }

        public Manufacturer GetOneByUrlName( Culture culture, string urlName )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllManufacturerFieldsWithLocale +
                " FROM Manufacturer LEFT JOIN  " +
                "     (SELECT * FROM ManufacturerLocale WHERE CultureID = @CultureID) AS L  " +
                "       ON Manufacturer.ManufacturerID = L.ManufacturerID " +
                "WHERE UrlName = @UrlName; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( urlName ) );

            if (table.Rows.Count > 0)
                return CreateOneManufacturerFromRow( culture, table.Rows[0] );
            else
                return NullManufacturer.Instance;
        }
        #endregion
    }
}
