using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class OptionGroupRepository : IOptionGroupRepository
    {
        #region Private

        private const string RepositoryName = "OptionGroupRepository";
        private const string RepositoryName1 = "GoogleFeedSpecMappingRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetLocalesSignature = "GetLocales";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string SearchOptionGroupSignature = "SearchOptionGroup";
        private const string GetAllSignature = "GetAll";
        private const string ProductIsOptionGroupSignature = "ProductIsOptionGroup";
        private const string IsUseStockSignature = "IsUseStock";

        private ICacheDependencyUtility _cacheDependency
            = new CacheDependencyUtility( new string[] { "OptionGroup", "OptionGroupLocale", "ProductOptionGroup" } );

        private DataCache _dataCache = new DataCache( RepositoryName );
        private DataCache _dataCache1 = new DataCache( RepositoryName1 );
        private OptionGroupRepositoryNoCache _repository = new OptionGroupRepositoryNoCache();

        #endregion

        #region Constructors

        public OptionGroupRepository()
        {
        }

        public OptionGroupRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public OptionGroup Save( OptionGroup optionGroup )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            OptionGroup newOptionGroup = _repository.Save( optionGroup );
            _dataCache.ClearRepositoryCache();
            _dataCache1.ClearRepositoryCache();
            return newOptionGroup;
        }

        public void Delete( string optionGroupID )
        {
            _repository.Delete( optionGroupID );
            _dataCache.ClearRepositoryCache();
            _dataCache1.ClearRepositoryCache();
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            _repository.DeleteLocalesByCultureID( cultureID );
            _dataCache.ClearRepositoryCache();
            _dataCache1.ClearRepositoryCache();
        }

        public OptionGroup GetOne( Culture culture, string optionGroupID )
        {
            OptionGroup result = (OptionGroup) _dataCache.GetValue( GetOneSignature, _cacheDependency, culture.CultureID, optionGroupID );

            if (result == null)
            {
                result = _repository.GetOne( culture, optionGroupID );
                _dataCache.SetValue( GetOneSignature, result, culture.CultureID, optionGroupID );
            }

            return result;
        }

        public IList<ILocale> GetLocales( string optionGroupID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache.GetValue( GetLocalesSignature, _cacheDependency, optionGroupID );

            if (list == null)
            {
                list = _repository.GetLocales( optionGroupID );
                _dataCache.SetValue( GetLocalesSignature, list, optionGroupID );
            }

            return list;
        }

        public IList<OptionGroup> GetAll(
           Culture culture
           )
        {
            IList<OptionGroup> result = (IList<OptionGroup>) _dataCache.GetValue(
                GetAllSignature, _cacheDependency, culture.CultureID
                );

            if (result == null)
            {
                result = new List<OptionGroup>();
                result = _repository.GetAll( culture );
                _dataCache.SetValue(
                    GetAllSignature,
                    result, culture.CultureID
                    );
            }
            return result;
        }

        public IList<OptionGroup> SearchOptionGroup(
           Culture culture,
           string sortBy,
           SearchFilter searchFilter,
           int startIndex,
           int endIndex,
           out int howManyItems
           )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchOptionGroupSignature, _cacheDependency,
                culture.CultureID,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchOptionGroup( culture, sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchOptionGroupSignature,
                    result,
                    culture.CultureID,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<OptionGroup>) result.First;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                   GetTableSchemaSignature, _cacheDependency
                   );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        public bool ProductIsOptionGroup( Culture culture, string productID )
        {
            object result = _dataCache.GetValue( ProductIsOptionGroupSignature, _cacheDependency, culture.CultureID, productID );
            if (result == null)
            {
                result = _repository.ProductIsOptionGroup( culture, productID );
                _dataCache.SetValue( ProductIsOptionGroupSignature, result, culture.CultureID, productID );
            }
            return ConvertUtilities.ToBoolean( result );
        }

        public bool IsUseStock( string productID, string optionGroupID )
        {
            object result = _dataCache.GetValue( IsUseStockSignature, _cacheDependency, productID, optionGroupID );
            if (result == null)
            {
                result = _repository.IsUseStock( productID, optionGroupID );
                _dataCache.SetValue( IsUseStockSignature, result, productID, optionGroupID );
            }
            return ConvertUtilities.ToBoolean( result );
        }

        #endregion
    }

}
