using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Web;
using System.Web.Security;
using System.Web.Services.Protocols;
using System.Xml;
using Vevo;
using Vevo.Domain;
using Vevo.Domain.Configurations;
using Vevo.Domain.Shipping;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;
using Vevo.WebAppLib;
using Vevo.Domain.Discounts;

namespace Vevo.Domain.Shipping.FedEx
{
    /// <summary>
    /// Summary description for FedExShippingGateway
    /// </summary>
    public class FedExShippingGateway : IShippingGateway
    {
        // Note: RateService.cs is created automatically from wsdl file
        // using VS command line:
        //      wsdl /namespace:Vevo.Domain.Shipping.FedEx XXXX.wsdl /out:XXXX.cs

        #region Private

        private const string ServiceListCacheName = "FedExShippingGateway_ServiceList";

        private ShippingOption _shippingOption;

        private void SetRequestAuthentication( RateRequest request )
        {
            request.WebAuthenticationDetail = new WebAuthenticationDetail();
            request.WebAuthenticationDetail.UserCredential = new WebAuthenticationCredential();
            request.WebAuthenticationDetail.UserCredential.Key =
                DataAccessContext.Configurations.GetValue( "RTShippingFedExKey" );
            request.WebAuthenticationDetail.UserCredential.Password =
                DataAccessContext.Configurations.GetValue( "RTShippingFedExPassword" );

            request.ClientDetail = new ClientDetail();
            request.ClientDetail.AccountNumber =
                DataAccessContext.Configurations.GetValue( "RTShippingFedExAccountNumber" );
            request.ClientDetail.MeterNumber =
                DataAccessContext.Configurations.GetValue( "RTShippingFedExMeterNumber" );
        }

        private void SetOrigin( RequestedShipment shipment )
        {
            shipment.Shipper = new Party();
            shipment.Shipper.Address = new Address();
            //shipment.Shipper.Address.StreetLines = new string[1] { "Sender Address Line 1" };
            //shipment.Shipper.Address.City = "Memphis";
            string countryCode = DataAccessContext.Configurations.GetValue( "RTShippingFedExMerchantCountry" );
            if (countryCode.Equals( "US", StringComparison.InvariantCultureIgnoreCase ))
                shipment.Shipper.Address.StateOrProvinceCode =
                    DataAccessContext.Configurations.GetValue( "RTShippingFedExMerchantState" );
            shipment.Shipper.Address.PostalCode =
                DataAccessContext.Configurations.GetValue( "RTShippingFedExMerchantZip" );
            shipment.Shipper.Address.CountryCode = countryCode;
        }

        private void SetDestination(
            RequestedShipment shipment,
            Vevo.Base.Domain.ShippingAddress shippingAddress )
        {
            shipment.Recipient = new Party();
            shipment.Recipient.Address = new Address();
            if (shippingAddress.State.Equals( "US", StringComparison.InvariantCultureIgnoreCase ))
                shipment.Recipient.Address.StateOrProvinceCode = shippingAddress.State;
            shipment.Recipient.Address.PostalCode = shippingAddress.Zip;
            shipment.Recipient.Address.CountryCode = shippingAddress.Country;

            // Set this option for FedEx Ground and FedEx Home Delivery
            shipment.Recipient.Address.Residential = shippingAddress.Residential;
            shipment.Recipient.Address.ResidentialSpecified = true;
        }

        private void SetPackageInformation( RequestedShipment shipment, double weight )
        {
            shipment.PackageDetail = RequestedPackageDetailType.PACKAGE_SUMMARY;
            shipment.PackageDetailSpecified = true;

            shipment.TotalWeight = new Weight();
            shipment.TotalWeight.Value = GetOrderWeight( weight );
            shipment.TotalWeight.Units = GetWeightUnit();
            //
            shipment.PackageCount = "1";
            //
            shipment.RequestedPackageLineItems = new RequestedPackageLineItem[1];
            shipment.RequestedPackageLineItems[0] = new RequestedPackageLineItem();

            //shipment.RequestedPackageLineItems[0].Dimensions = new Dimensions(); // package dimensions, applies to each package 
            //shipment.RequestedPackageLineItems[0].Dimensions.Length = "10";
            //shipment.RequestedPackageLineItems[0].Dimensions.Width = "10";
            //shipment.RequestedPackageLineItems[0].Dimensions.Height = "3";
            //shipment.RequestedPackageLineItems[0].Dimensions.Units = LinearUnits.IN;

            //shipment.RequestedPackageLineItems[0].InsuredValue = new Money(); // insured value
            //shipment.RequestedPackageLineItems[0].InsuredValue.Amount = 100;
            //shipment.RequestedPackageLineItems[0].InsuredValue.Currency = "USD";
        }

        private void SetRequestedCarrierCodes( RateRequest request )
        {
            // Insert the Carriers you would like to see the rates for
            request.CarrierCodes = new CarrierCodeType[3];
            request.CarrierCodes[0] = CarrierCodeType.FDXE;
            request.CarrierCodes[1] = CarrierCodeType.FDXG;
            request.CarrierCodes[2] = CarrierCodeType.FXSP;
        }

        private decimal GetOrderWeight( double weight )
        {
            decimal result = (decimal) weight;

            if (DataAccessContext.Configurations.GetBoolValue( "RTShippingFedExUseMinOrderWeight" ))
            {
                decimal minimumWeight =
                    DataAccessContext.Configurations.GetDecimalValue( "RTShippingFedExMinOrderWeight" );
                if (minimumWeight > (decimal) weight)
                    result = minimumWeight;
            }

            return result;
        }

        private WeightUnits GetWeightUnit()
        {
            return EnumUtilities<WeightUnits>.Parse(
                DataAccessContext.Configurations.GetValue( "WeightUnit" ) );
        }

        private DropoffType GetDropoffType()
        {
            return EnumUtilities<DropoffType>.Parse(
                DataAccessContext.Configurations.GetValue( "RTShippingFedExDropoffType" ) );
        }

        private RateRequest CreateRequest(
            double weight,
            Vevo.Base.Domain.ShippingAddress shippingAddress )
        {
            RateRequest request = new RateRequest();
            SetRequestAuthentication( request );

            //// This is a pass-through value and could be anything
            //request.TransactionDetail = new TransactionDetail();
            //request.TransactionDetail.CustomerTransactionId = "Rate Available Request";

            request.Version = new VersionId();

            // Request transit time information (date & time to be delivered)
            request.ReturnTransitAndCommit = true;
            request.ReturnTransitAndCommitSpecified = true;

            request.RequestedShipment = new RequestedShipment();

            request.RequestedShipment.ShipTimestamp = DateTime.Now; // Shipping date and time
            request.RequestedShipment.ShipTimestampSpecified = true;

            request.RequestedShipment.DropoffTypeSpecified = true;
            request.RequestedShipment.DropoffType = GetDropoffType();

            request.RequestedShipment.RateRequestTypes = new RateRequestType[1];
            request.RequestedShipment.RateRequestTypes[0] = RateRequestType.ACCOUNT;

            SetOrigin( request.RequestedShipment );
            SetDestination( request.RequestedShipment, shippingAddress );

            SetPackageInformation( request.RequestedShipment, weight );

            SetRequestedCarrierCodes( request );

            return request;
        }

        private bool IsDataInConfigValueList( string configName, string data )
        {
            string[] valueList = DataAccessContext.Configurations.GetValueList( configName );
            for (int i = 0; i < valueList.Length; i++)
            {
                if (valueList[i] == data)
                    return true;
            }

            return false;
        }

        private bool IsShippingOptionEnabled( RateReplyDetail replyDetail )
        {
            string serviceName;
            string serviceCode;
            GetServiceDetails( replyDetail.ServiceType, out serviceName, out serviceCode );

            return IsDataInConfigValueList( "RTShippingFedExServiceEnabled", serviceCode );
        }

        private bool IsFreeService( string serviceCode, decimal subtotal )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "RTShippingFedExFreeShipping" )
                && subtotal > DataAccessContext.Configurations.GetDecimalValue( "RTShippingFedExFreeShippingCost" ))
            {
                return IsDataInConfigValueList( "RTShippingFedExFreeShippingCostService", serviceCode );
            }
            else
            {
                return false;
            }
        }

        private bool IsAllowedFreeShippingCostItem( string serviceCode, bool isContainFreeShippingItem )
        {
            if (isContainFreeShippingItem)
            {
                string[] serviceList = DataAccessContext.Configurations.GetValueList( "RTShippingFedExServiceAllowedSetFree" );

                foreach (string service in serviceList)
                {
                    if (service == serviceCode)
                        return true;
                }
            }
            return false;
        }

        private DataRow GetServiceDataRow( ServiceType serviceType )
        {
            DataTable table = ReadServiceFile();
            foreach (DataRow row in table.Rows)
            {
                if (row["Name"].ToString() == serviceType.ToString())
                    return row;
            }

            return null;
        }

        private void GetServiceDetails(
            ServiceType serviceType,
            out string serviceName,
            out string serviceCode )
        {
            DataRow row = GetServiceDataRow( serviceType );
            if (row != null)
            {
                serviceName = row["DisplayName"].ToString().Trim();
                serviceCode = row["Code"].ToString().Trim();
            }
            else
            {
                serviceName = String.Empty;
                serviceCode = String.Empty;
            }
        }

        private ShippingChoice CreateShippingOption(
            RateReplyDetail replyDetail,
            decimal subtotal,
             bool isContainFreeShippingItem,
            Coupon coupon )
        {
            string serviceName;
            string serviceCode;
            GetServiceDetails( replyDetail.ServiceType, out serviceName, out serviceCode );

            decimal price;
            if (IsFreeService( serviceCode, subtotal )
                || IsAllowedFreeShippingCostItem( serviceCode, isContainFreeShippingItem )
                || IsAllowedUseCouponFreeShippingCost( serviceCode, coupon ))
                price = 0;
            else
                price =
                    replyDetail.RatedShipmentDetails[0].ShipmentRateDetail.TotalNetCharge.Amount
                    + DataAccessContext.Configurations.GetDecimalValue( "RTShippingFedExMarkup" );

            return new ShippingChoice(
                serviceName,
                price,
                _shippingOption.GetHandlingFee(
                    DataAccessContext.Configurations.GetBoolValue( "HandlingFeeEnabled" ), price ) );
        }

        private string CreateErrorData( RateReply reply )
        {
            if (reply.Notifications.Length > 0)
            {
                Notification notification = reply.Notifications[0];

                switch (ConvertUtilities.ToInt32( notification.Code ))
                {
                    case 505:
                        return
                            "Package weight or dimension is invalid. " +
                            "You may have chosen too many items in this order.";

                    default:
                        return
                            notification.Severity.ToString() + " " +
                            notification.Code + ": " +
                            notification.Message;
                }
            }

            return String.Empty;
        }

        private void PopulateShippingChoices(
            IList<ShippingChoice> shippingChoices,
            RateReply reply,
            decimal subtotal,
             bool isContainFreeShippingItem,
            Coupon coupon )
        {
            foreach (RateReplyDetail replyDetail in reply.RateReplyDetails)
            {
                if (IsShippingOptionEnabled( replyDetail ))
                {
                    ShippingChoice shippingOption = CreateShippingOption( replyDetail, subtotal, isContainFreeShippingItem, coupon );
                    shippingChoices.Add( shippingOption );
                }
            }
        }

        private bool IsAllowedUseCouponFreeShippingCost( string serviceCode, Coupon coupon )
        {
            if (coupon.DiscountType == Coupon.DiscountTypeEnum.FreeShipping)
            {
                string[] serviceList = DataAccessContext.Configurations.GetValueList(
                    "RTShippingFedExServiceAllowedUseCouponFree" );

                foreach (string service in serviceList)
                {
                    if (service == serviceCode)
                        return true;
                }

            }
            return false;
        }

        #endregion


        #region Constructors

        public FedExShippingGateway( ShippingOption shippingOption )
        {
            _shippingOption = shippingOption;
        }

        #endregion


        #region Public Methods

        public static DataTable ReadServiceFile()
        {
            DataTable serviceList = (DataTable) HttpContext.Current.Cache[ServiceListCacheName];
            if (serviceList == null)
            {
                DataSet ds = new DataSet();
                string pathFile = HttpContext.Current.Server.MapPath( "~/DataFiles/Shipping/FedExService.xml" );
                if (File.Exists( pathFile ))
                    ds.ReadXml( pathFile );
                else
                    throw new Exception( "File Not Found : " + pathFile );

                serviceList = ds.Tables[0];

                HttpContext.Current.Cache[ServiceListCacheName] = serviceList;
            }
            return serviceList;
        }

        #endregion


        #region IShippingGateway Members

        public IList<ShippingChoice> RequestShippingChoices(
            Vevo.Base.Domain.ShippingAddress shippingAddress,
            double weight,
            decimal subtotal,
            bool isContainFreeShippingItem,
            Coupon coupon,
            out string errorMessage,
            out string restriction )
        {
            errorMessage = String.Empty;
            restriction = String.Empty;

            IList<ShippingChoice> shippingChoices = new List<ShippingChoice>();

            RateRequest request = CreateRequest( weight, shippingAddress );
            RateService service = new RateService();
            service.Url = DataAccessContext.Configurations.GetValue( "RTShippingFedExUrl" );
            try
            {
                RateReply reply = service.getRates( request );

                if ((reply.HighestSeverity != NotificationSeverityType.ERROR) &&
                    (reply.HighestSeverity != NotificationSeverityType.FAILURE) &&
                    (reply.HighestSeverity != NotificationSeverityType.WARNING))
                {
                    PopulateShippingChoices( shippingChoices, reply, subtotal, isContainFreeShippingItem, coupon );
                }
                else
                {
                    errorMessage = CreateErrorData( reply );
                }
            }
            catch (SoapException e)
            {
                SaveLogFile.SaveLog( e );
                if (!DataAccessContext.Configurations.GetBoolValue( "ShowDetailAjaxErrorMessage" ))
                    errorMessage = e.Detail.InnerText;
                else
                    throw;
            }
            catch (Exception e)
            {
                SaveLogFile.SaveLog( e );
                if (!DataAccessContext.Configurations.GetBoolValue( "ShowDetailAjaxErrorMessage" ))
                    errorMessage = e.Message;
                else
                    throw;
            }

            return shippingChoices;
        }

        #endregion

    }
}
