using System;
using System.Data;
using System.Collections;
using System.Configuration;
using System.Text;
using System.Text.RegularExpressions;

using Vevo.Domain.Exceptions;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.Orders
{
    /// <summary>
    /// Summary description for OptionItemValueCollection
    /// </summary>
    [Serializable]
    public class OptionItemValueCollection : CollectionBase
    {
        #region Private

        private const string ItemSeparator = ",";
        private const string OptionRegExString = @"(?<ID>\d+)(""(?<Text>([^""]*("""")?)*)"")?" + ItemSeparator;

        private string CreateFormatOptionText( string optionItemID, string details )
        {
            return optionItemID + "\"" + details.Replace( "\"", "\"\"" ) + "\"";
        }

        private bool CompareCondition( OptionItemValue source, OptionItemValue compare )
        {
            // For Option Input List, "Details" field will contain quantity (e.g. 1).
            // Therefore, the comparison may not be the same even though OptionItemID and
            // UseStock are the same.
            if (source.GroupType == OptionGroup.OptionGroupType.InputList &&
                compare.GroupType == OptionGroup.OptionGroupType.InputList)
            {
                return (source.OptionItemID == compare.OptionItemID &&
                    source.UseStock == compare.UseStock);
            }
            else
            {
                return (source.Details == compare.Details &&
                    //source.GroupType == compare.GroupType &&
                    source.OptionItemID == compare.OptionItemID &&
                    source.UseStock == compare.UseStock);
            }
        }

        private bool IsUseStock( Product product, OptionGroup optionGroup )
        {
            foreach (ProductOptionGroup productOptionGroup in product.ProductOptionGroups)
            {
                if (productOptionGroup.OptionGroupID == optionGroup.OptionGroupID)
                    return productOptionGroup.UseStock;
            }

            // Return false instead of throwing exception
            return false;

            // Disable exception for now.
            // The reason is because when product in Wish List has removed association with an option grouop,
            // this error below will be thrown.
            // May need to consider whether to keep "UseStock" property in OptionItemValue class.

            //throw new DomainException(
            //    "Invalid option group ID " + optionGroup.OptionGroupID 
            //    + " for product ID " + product.ProductID );
        }

        #endregion


        #region Constructors

        public OptionItemValueCollection()
        {
        }

        public OptionItemValueCollection( OptionItemValue[] optionItemValue )
        {
            foreach (OptionItemValue item in optionItemValue)
            {
                this.Add( item );
            }
        }

        public OptionItemValueCollection( Culture culture, string encodedOptions, string productID)
        {
            if (!encodedOptions.EndsWith( ItemSeparator ))
                encodedOptions += ItemSeparator;

            Product product = DataAccessContext.ProductRepository.GetOne( culture, productID, Stores.NullStore.Null.StoreID );

            Regex regex = new Regex( OptionRegExString );
            MatchCollection matches = regex.Matches( encodedOptions );

            foreach (Match match in matches)
            {
                string optionItemID = match.Groups["ID"].Value;

                OptionItem optionItem = DataAccessContext.OptionItemRepository.GetOne( culture, optionItemID );
                OptionGroup optionGroup = DataAccessContext.OptionGroupRepository.GetOne(
                    culture, optionItem.OptionGroupID );

                string text =
                    String.IsNullOrEmpty( match.Groups["Text"].Value )
                        ? "" : match.Groups["Text"].Value;

                OptionItemValue itemValue = new OptionItemValue(
                    optionItem, optionGroup.Type, text, IsUseStock( product, optionGroup ) );

                this.Add( itemValue );
            }
        }

        #endregion


        #region Public Properties

        // Return null object
        public static OptionItemValueCollection Null
        {
            get { return NullOptionItemValueCollection.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        #endregion


        #region Public Methods

        public override string ToString()
        {
            string result = String.Empty;

            foreach (OptionItemValue optionValue in this)
            {
                if (optionValue.GroupType == OptionGroup.OptionGroupType.Text)
                    result += CreateFormatOptionText( optionValue.OptionItemID, optionValue.Details )
                        + ItemSeparator;
                else
                    result += optionValue.OptionItemID + ItemSeparator;
            }

            return result;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        public override bool Equals( object obj )
        {
            OptionItemValueCollection compareList = (OptionItemValueCollection) obj;

            if (this.Count != compareList.Count)
            {
                return false;
            }
            else
            {
                bool equal = true;

                foreach (OptionItemValue source in this)
                {
                    bool result = false;
                    foreach (OptionItemValue compare in compareList)
                    {
                        if (CompareCondition( source, compare ))
                        {
                            result = true;
                            break;
                        }
                    }

                    if (!result)
                    {
                        equal = false;
                        break;
                    }
                }

                return equal;
            }
        }

        public virtual void Add( OptionItemValue optionItem )
        {
            this.InnerList.Add( optionItem );
        }

        public OptionItemValue this[int index]
        {
            get
            {
                return (OptionItemValue) this.InnerList[index];
            }
            set
            {
                this.InnerList[index] = value;
            }
        }

        // This is shallow copy
        public OptionItemValueCollection Clone()
        {
            OptionItemValueCollection collection = new OptionItemValueCollection();

            foreach (OptionItemValue option in this)
            {
                collection.Add( option );
            }

            return collection;
        }

        public string[] ConvertToIDArray()
        {
            ArrayList list = new ArrayList();

            foreach (OptionItemValue optionItem in this)
            {
                list.Add( optionItem.OptionItemID );
            }

            string[] result = new string[list.Count];
            list.CopyTo( result );
            return result;
        }

        public string[] GetUseStockOptionItemIDs()
        {
            ArrayList list = new ArrayList();

            foreach (OptionItemValue optionItem in this)
            {
                if (optionItem.UseStock)
                {
                    list.Add( optionItem.OptionItemID );
                }
            }

            string[] result = new string[list.Count];
            list.CopyTo( result );
            return result;
        }

        public OptionItemValueCollection GetUseStockOptionItems()
        {
            OptionItemValueCollection collection = new OptionItemValueCollection();

            foreach (OptionItemValue optionItem in this)
            {
                if (optionItem.UseStock)
                    collection.Add( optionItem );
            }

            return collection;
        }

        public OptionItemValueCollection GetOptionItemsByGroupType( OptionGroup.OptionGroupType groupType )
        {
            OptionItemValueCollection collection = new OptionItemValueCollection();

            foreach (OptionItemValue optionItem in this)
            {
                if (optionItem.GroupType == groupType)
                    collection.Add( optionItem );
            }

            return collection;
        }

        public OptionItemValueCollection GetOptionItemsNotInGroupType( OptionGroup.OptionGroupType groupType )
        {
            OptionItemValueCollection collection = new OptionItemValueCollection();

            foreach (OptionItemValue optionItem in this)
            {
                if (optionItem.GroupType != groupType)
                    collection.Add( optionItem );
            }

            return collection;
        }

        public virtual decimal GetOptionPrice( decimal productPrice )
        {
            decimal optionPrice = 0;

            foreach (object optionItemValue in InnerList)
                optionPrice += ((OptionItemValue) optionItemValue).OptionItem.GetOptionPrice( productPrice );

            return optionPrice;
        }

        public virtual double GetOptionWeight()
        {
            double result = 0;

            foreach (OptionItemValue optionItemValue in InnerList)
            {
                result += ConvertUtilities.ToDouble( optionItemValue.OptionItem.WeightToAdd );
            }

            return result;
        }

        public string GetUploadedListString( Culture culture, Currency currency )
        {
            StringBuilder sb = new StringBuilder();

            bool isFirst = true;
            foreach (OptionItemValue optionItemValue in InnerList)
            {
                if (optionItemValue.TempUploadedFile != null)
                {
                    if (!isFirst)
                        sb.Append( ", " );

                    string optionName = optionItemValue.GetDisplayedName( culture, currency );
                    string[] filenames = optionName.Split( new string[] { " (" }, StringSplitOptions.None );
                    string outputFilename = String.Empty;
                    if (filenames[filenames.Length - 1].EndsWith( ")" ))
                    {
                        for (int i = 0; i < filenames.Length - 1; i++)
                            outputFilename += filenames[i];
                    }
                    else
                    {
                        outputFilename = optionName;
                    }
                    sb.Append( optionName.Replace( ',', ' ' ) );

                    isFirst = false;
                }
            }

            return sb.ToString();
        }

        #endregion

    }
}
